'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// function to generates error as you see.
var errUnit = function errUnit(errMsg) {
  return new Error('\x1B[33m' + errMsg + '\x1B[0m');
};

// since all the validation happens after opening the file.
var closeReturnError = function closeReturnError(fd, errMsg, cb) {
  return _fs2.default.close(fd, function () {
    return cb(errUnit(errMsg));
  });
};

var cbMsg = 'Invalid callback function.';
var arrOfRegMsg = 'Invalid array type';
var arrOfRegEmptyMsg = 'Empty array of objects';
var regMsg = 'Invalid regex at: ';

// gets the regex length
var regLen = function regLen(regex) {
  return regex.length;
};

// check if input is string
var isStr = function isStr(str) {
  return typeof str === 'string';
};

// check if input is regex
var isReg = function isReg(reg) {
  return (typeof reg === 'undefined' ? 'undefined' : _typeof(reg)) === 'object';
};

/**
 *
 * @param {Object} opt - options
 * @property {string} - path, directory
 * @property {array} - requestز array contains the regex
 * @property {string} - encoding, type of encoding used for reading
 * @property {number} - rounds, chunk boundaries
 *
 * @callback {error~object}
 * @property {boolean} - isFound
 * @property {string|object} - reg
 * @property {string} - match,
 *
 */
var find = function find(opt, cb) {
  var opts = opt || {};
  // validate callback, else every error will be handled in callback error
  if (typeof cb !== 'function') throw errUnit(cbMsg);
  // validate file.
  return _fs2.default.open(opts.path, 'r', function (openErr, fd) {
    if (openErr) return cb(openErr);
    // validate arrayOfReg
    if (!(opts.request instanceof Array)) return closeReturnError(fd, arrOfRegMsg, cb);
    // validate arrayOfReg length
    var arrayLen = regLen(opts.request);
    if (arrayLen === 0) return closeReturnError(fd, arrOfRegEmptyMsg, cb);
    // validate the objects inside.
    for (var i = 0; i < arrayLen; i += 1) {
      // validate regMsg in find
      if (!isReg(opts.request[i]) && !isStr(opts.request[i])) {
        return closeReturnError(fd, regMsg + opts.request[i], cb);
      }
    }
    // Everything is good ****************************************************
    // create array of flags
    var isFlags = [];
    // init arry with default false.
    for (var _i = 0; _i < arrayLen; _i += 1) {
      isFlags[_i] = null;
    } // flags
    var isFoundAll = false;
    var readStream = _fs2.default.createReadStream(opts.path, {
      encoding: opts.encoding || 'utf8'
    });
    readStream.on('error', function (readErr) {
      return closeReturnError(fd, readErr, cb);
    });
    var combinedChunkQueue = []; // chunck holder
    var combinedChunk = void 0; // combined chunk
    var maxRounds = opts.join || 3; // max rounds allowed to hold chunk
    var currentRound = 0;
    readStream.on('data', function (chunk) {
      // handling combined chunk index.
      if (currentRound === maxRounds) {
        // reset index
        currentRound = 0;
        // dump the first element in queue
        combinedChunkQueue.shift();
      } else {
        currentRound += 1;
        // add new chunck
        combinedChunkQueue.push(chunk);
      }
      // flush combined chunk
      combinedChunk = null;
      // compose combined chunk
      for (var j = 0; j < combinedChunkQueue.length; j += 1) {
        combinedChunk += combinedChunkQueue[j];
      } // matching process
      for (var _i2 = 0; _i2 < arrayLen; _i2 += 1) {
        if (isFlags[_i2] === null) isFlags[_i2] = combinedChunk.match(opts.request[_i2]);
      }
      if (!isFlags.includes(null)) isFoundAll = true;
      if (isFoundAll) {
        combinedChunk = null;
        readStream.destroy();
        readStream.emit('end');
      }
    });
    return readStream.on('end', function () {
      return _fs2.default.close(fd, function () {
        var report = [];
        for (var _i3 = 0; _i3 < arrayLen; _i3 += 1) {
          report[_i3] = {
            isFound: isFlags[_i3] !== null,
            reg: opts.request[_i3],
            match: isFlags[_i3] && isFlags[_i3][0]
          };
        }
        return cb(null, report);
      });
    });
  });
};

exports.default = find;